"""
This program initializes a one-wire bus on a specified pin (IO21) 
and scans for DS18X20 temperature sensors connected to the bus. 
For each detected sensor, the program retrieves its ROM address 
and creates a DS18X20 object. 
It sets each sensor to a resolution of 12-bit and prints the detected 
sensors' ROM addresses.

The temperature is measure from each sensore in every second
(plus teh delay required for the conversion) and printed out.

Steps:
1. Initialize the one-wire bus on board pin D1.
2. Scan for devices on the bus.
3. Create a list of DS18X20 objects for each detected sensor.
4. Print the ROM address of each detected sensor.
5. In the main loop, read the temperature from all sensors and 
   print the sensor index and temperature every second.

Usage:
This program is intended for use with CircuitPython and compatible hardware.
"""

import time
import board
from adafruit_onewire.bus import OneWireBus
from adafruit_ds18x20 import DS18X20

# Initialize one-wire bus on board pin D1
ow_bus = OneWireBus(board.IO21)

# Scan for devices on the bus
devices = ow_bus.scan()

# Create a list of DS18X20 objects
sensors = []
for i, d in enumerate(devices):
    print("Found sensor {} ROM=0x{}".format(i, d.rom.hex()))
    sensor = DS18X20(ow_bus, d)
    sensor.resolution = 12
    sensors.append((i, sensor))

# Main loop to print the temperature from all sensors every second
while True:
    print()  # Print an empty line for better output separation
    for i, sensor in sensors:
        conversion_delay = sensor.start_temperature_read()
        conversion_ready_at = time.monotonic() + conversion_delay
        while time.monotonic() < conversion_ready_at:
            time.sleep(0.1)
        print(f"Sensor {i}: {sensor.read_temperature():0.3f}C")
    time.sleep(1.0)
