"""
Scan Onewire bus and read the ROM of DS18B20 sensors

This program is a slightly modified version of the official example onewire_simpletest.py 
(link: https://docs.circuitpython.org/projects/onewire/en/latest/examples.html)
runs on an ESP32-C3 Supermini board and reads the 
ROM values of DS18B20 temperature sensors connected to a 1-Wire bus. 
The program uses the adafruit_onewire library to interact with the sensors.

Requirements:
- CircuitPython installed on the ESP32-C3 Supermini board.
- adafruit_onewire library installed.
- DS18B20 temperature sensors connected to the specified pin on the board (IO21 in this example).

The program performs the following steps:
1. Initializes the 1-Wire bus.
2. Resets the bus and checks for the presence of devices.
3. Scans the bus for connected devices and retrieves their ROM values.
4. Prints the ROM values, family codes, and serial numbers of the found devices.

Typical result:
---------------
code.py output:
Resetting bus...OK.
Scanning for devices...OK.
Found 2 device(s).
 
Device   0
    CRC check passed!
    ROM=0x28ee47be2c1602cd    Family=0x28 CRC=0xcd
    Serial Number = 0xee 0x47 0xbe 0x2c 0x16 0x02 
 
Device   1
    CRC check passed!
    ROM=0x28eed98d2e1601b5    Family=0x28 CRC=0xb5
    Serial Number = 0xee 0xd9 0x8d 0x2e 0x16 0x01 

Code done running.
"""

import board
from adafruit_onewire.bus import OneWireBus

# Create the 1-Wire Bus
# Use whatever pin you've connected to on your board
ow_bus = OneWireBus(board.IO21)

# Reset and check for presence pulse.
# This is basically - "is there anything out there?"
print("Resetting bus...", end="")
if ow_bus.reset():
    print("OK.")
else:
    raise RuntimeError("Nothing found on bus.")

# Run a scan to get all of the device ROM values
print("Scanning for devices...", end="")
devices = ow_bus.scan()
print("OK.")
print("Found {} device(s).".format(len(devices)))

# For each device found, print out some info
for i, d in enumerate(devices):
    print(" ")
    print("Device {:>3}".format(i))
    if (d.crc == ow_bus.crc8(d.rom[:-1])):
        print("    CRC check passed!")
    else:
        print("    CRC check FAILED!")    
    print("    ROM=0x{}    Family=0x{:02x} CRC=0x{:02x}".format(d.rom.hex(), d.family_code, d.crc))    
    print("    Serial Number = ", end="")
    for byte in d.serial_number:
        print("0x{:02x} ".format(byte), end="")
    print()
