# SPDX-FileCopyrightText: 2025 I. Cserny
# SPDX-FileCopyrightText: 2018 Tony DiCola for Adafruit Industries
# SPDX-FileCopyrightText: 2018 ladyada for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`st7585`
====================================================

A display control library for Shanyan ST7585-based monochrome displays.

This library is a slightly modified version of the Adafruit_PCD8544 library 
<https://github.com/adafruit/Adafruit_CircuitPython_PCD8544>

* Author(s): ladyada, Tony DiCola, I. Cserny

Implementation Notes
--------------------

1. Due to the different characteristics of the display controller chips, the addressing mode had to be changed.
2. The ST7585 has an additional pixel row (page 8) where fixed graphics icons can be displayed. The data sent to this row does not define the graphics itself but controls the enable (1) or disable (0) state of each pixel in the icons.
3. To handle the icon row, the following member functions were added to the ST7585 class:
   - `show_icons()`: Displays the entire icon row.
   - `hide_icons()`: Hides the icon row.
   - `set_icons(bytearray b)`: Allows custom configuration of the icon row.

**Hardware:**

* `Shanyan ST7585 Display <https://www.alldatasheet.com/datasheet-pdf/pdf/326181/SITRONIX/ST7585.html>`_

**Software and Dependencies:**

* Adafruit CircuitPython 9.x firmware for the supported boards:
  https://github.com/adafruit/circuitpython/releases

* Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice

* Adafruit FrameBuffer library: https://github.com/adafruit/Adafruit_CircuitPython_FrameBuffer
"""

import time
from micropython import const
from adafruit_bus_device import spi_device

try:
    import framebuf
except ImportError:
    import adafruit_framebuf as framebuf

try:
    from typing import Optional
    from digitalio import DigitalInOut
    from busio import SPI
except ImportError:
    pass

__version__ = "1.0.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_PCD8544.git"

_LCDWIDTH = const(96)
_LCDHEIGHT = const(64)
_ST7585_POWERDOWN = const(0x04)
_ST7585_ENTRYMODE = const(0x02)
_ST7585_EXTENDEDINSTRUCTION = const(0x01)
_ST7585_DISPLAYBLANK = const(0x0)
_ST7585_DISPLAYNORMAL = const(0x4)
_ST7585_DISPLAYALLON = const(0x1)
_ST7585_DISPLAYINVERTED = const(0x5)
_ST7585_FUNCTIONSET = const(0x20)
_ST7585_DISPLAYCONTROL = const(0x08)
_ST7585_SETYADDR = const(0x40)
_ST7585_SETXADDR = const(0x80)
_ST7585_SETTEMP = const(0x04)
_ST7585_SETBIAS = const(0x10)
_ST7585_SETVOP = const(0x80)


class ST7585(framebuf.FrameBuffer):
    """Shanyan ST7585-based LCD display."""

    # pylint: disable=too-many-instance-attributes

    def __init__(
        self,
        spi: SPI,
        dc_pin: DigitalInOut,
        cs_pin: DigitalInOut,
        reset_pin: Optional[DigitalInOut] = None,
        *,
        contrast: int = 80,
        bias: int = 4,
        baudrate: int = 1000000
    ) -> None:
        self._dc_pin = dc_pin
        dc_pin.switch_to_output(value=False)

        self.spi_device = spi_device.SPIDevice(spi, cs_pin, baudrate=baudrate)

        self._reset_pin = reset_pin
        if reset_pin:
            reset_pin.switch_to_output(value=True)

        self.buffer = bytearray((_LCDHEIGHT // 8) * _LCDWIDTH)
        super().__init__(self.buffer, _LCDWIDTH, _LCDHEIGHT)

        self._contrast = None
        self._bias = None
        self._invert = False

        self.reset()
        # Set LCD bias.
        self.bias = bias
        self.contrast = contrast

    def reset(self) -> None:
        """Reset the display"""
        if self._reset_pin:
            # Toggle RST low to reset.
            self._reset_pin.value = False
            time.sleep(0.5)
            self._reset_pin.value = True
            time.sleep(0.5)

    def write_cmd(self, cmd: int) -> None:
        """Send a command to the SPI device"""
        self._dc_pin.value = 0
        with self.spi_device as spi:
            spi.write(bytearray([cmd]))  # pylint: disable=no-member

    def extended_command(self, cmd: int) -> None:
        """Send a command in extended mode"""
        # Set extended command mode
        self.write_cmd(_ST7585_FUNCTIONSET | _ST7585_EXTENDEDINSTRUCTION)
        self.write_cmd(cmd)
        # Set normal display mode.
        self.write_cmd(_ST7585_FUNCTIONSET)
        self.write_cmd(_ST7585_DISPLAYCONTROL | _ST7585_DISPLAYNORMAL)

    def show(self) -> None:
        """Write out the frame buffer via SPI"""
        for page in range(_LCDHEIGHT // 8):
            self.write_cmd(_ST7585_SETYADDR | (7 - page))  # Set Y address for each page in reverse order
            self.write_cmd(_ST7585_SETXADDR | 0)  # Set X address to 0
            self._dc_pin.value = True  # Set to data mode
            with self.spi_device as spi:
                spi.write(self.buffer[page * _LCDWIDTH : (page + 1) * _LCDWIDTH])  # Write buffer data for each page        

    def show_icons(self) -> None:
        """Write icons to the display via SPI"""
        self.write_cmd(_ST7585_SETYADDR | 8)  # Set Y address to 8 for icon row
        self.write_cmd(_ST7585_SETXADDR | 0)  # Set X address to 0
        self._dc_pin.value = True  # Set to data mode
        with self.spi_device as spi:
            spi.write(bytearray([0xFF] * 96))  # Write 96 bytes of 0xFF

    def hide_icons(self) -> None:
        """Write blank icons to the display via SPI"""
        self.write_cmd(_ST7585_SETYADDR | 8)  # Set Y address to 8 for icon row
        self.write_cmd(_ST7585_SETXADDR | 0)  # Set X address to 0
        self._dc_pin.value = True  # Set to data mode
        with self.spi_device as spi:
            spi.write(bytearray([0x00] * 96))  # Write 96 bytes of 0x00

    def set_icons(self, icons: bytearray) -> None:
        """Write custom icons to the display via SPI"""
        if len(icons) != 96:
            raise ValueError("Icons bytearray must be exactly 96 bytes long")
        self.write_cmd(_ST7585_SETYADDR | 8)  # Set Y address to 8 for icon row
        self.write_cmd(_ST7585_SETXADDR | 0)  # Set X address to 0
        self._dc_pin.value = True  # Set to data mode
        with self.spi_device as spi:
            spi.write(icons)  # Write custom icons to the display


    @property
    def invert(self) -> bool:
        """Whether the display is inverted, cached value"""
        return self._invert

    @invert.setter
    def invert(self, val: bool) -> None:
        """Set invert on or normal display on"""
        self._invert = val
        self.write_cmd(_ST7585_FUNCTIONSET)
        if val:
            self.write_cmd(_ST7585_DISPLAYCONTROL | _ST7585_DISPLAYINVERTED)
        else:
            self.write_cmd(_ST7585_DISPLAYCONTROL | _ST7585_DISPLAYNORMAL)

    @property
    def contrast(self) -> int:
        """The cached contrast value"""
        return self._contrast

    @contrast.setter
    def contrast(self, val: int) -> None:
        """Set contrast to specified value (should be 0-127)."""
        self._contrast = max(0, min(val, 0x7F))  # Clamp to values 0-0x7f
        self.extended_command(_ST7585_SETVOP | self._contrast)

    @property
    def bias(self) -> int:
        """The cached bias value"""
        return self._bias

    @bias.setter
    def bias(self, val: int) -> None:
        """Set display bias"""
        self._bias = val
        self.extended_command(_ST7585_SETBIAS | self._bias)
